package com.epson.SampleCode;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.InputStream;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.res.Configuration;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.RadioGroup;
import android.widget.RadioGroup.OnCheckedChangeListener;
import android.widget.Spinner;
import android.widget.TextView;

import com.epson.EpsonCom.CallbackInterface;
import com.epson.EpsonCom.EpsonCom;
import com.epson.EpsonCom.EpsonCom.ALIGNMENT;
import com.epson.EpsonCom.EpsonCom.BITDEPTH;
import com.epson.EpsonCom.EpsonCom.ERROR_CODE;
import com.epson.EpsonCom.EpsonCom.FONT;
import com.epson.EpsonCom.EpsonCom.IMAGEFORMAT;
import com.epson.EpsonCom.EpsonCom.IMAGEPROCESSING;
import com.epson.EpsonCom.EpsonCom.MICR_FONT;
import com.epson.EpsonCom.EpsonCom.PAPERSIDE;
import com.epson.EpsonCom.EpsonCom.PORT_TYPE;
import com.epson.EpsonCom.EpsonComASBStatus;
import com.epson.EpsonCom.EpsonComCallbackInfo;
import com.epson.EpsonCom.EpsonComDevice;
import com.epson.EpsonCom.EpsonComDeviceParameters;

public class SampleCodeforEpsonComActivity extends Activity implements CallbackInterface 
{
    //EpsonCom objects
    EpsonComDevice m_Device;
    EpsonComDeviceParameters m_DeviceParameters;
    EpsonComASBStatus m_ASBData;
    
    //Status Thread
 	Thread m_statusThread;
 	boolean m_bStatusThreadStop;

    //progress dialog
 	ProgressDialog m_progressDialog;
 	

   /**************************************************************************************
    * Called when the activity is first created.
    **************************************************************************************/
    @Override
    public void onCreate(Bundle savedInstanceState) 
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        try
        {
        	m_progressDialog=null;
        	
        	//attach a listener to the print station radio group 
   	        RadioGroup radioGroupPrintStation =(RadioGroup)findViewById(R.id.radioGroupPrintStation);
	    	radioGroupPrintStation.setOnCheckedChangeListener(new OnCheckedChangeListener()
	    	{
	    	    public void onCheckedChanged(RadioGroup group, int checkedId)
	    	    {
	    	    	printStationRadioGroupClicked(checkedId);
	    	    }
	    	});    	

        	//fill spinner with data
	        Spinner spinner = (Spinner) findViewById(R.id.spinnerPortType);
	        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this, R.array.porttypes_array, android.R.layout.simple_spinner_item);
	        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
	        spinner.setAdapter(adapter);	    	
	    	
        	//initialize EpsonCom
        	m_Device = new EpsonComDevice();
        	m_DeviceParameters = new EpsonComDeviceParameters();
        	
	        //register myself as callback
	        m_Device.registerCallback(this);
        	
        	//create and run status thread
        	createAndRunStatusThread(this);
        	
        }
        catch(Exception e)
        {
			messageBox(this, "Exception: " + e.toString() + " - " + e.getMessage(), "onCreate Error");
        }
    }
    
    
    /**************************************************************************************
     * onConfigurationChanged
     * This is called when something changes, e.g. the orientation of the tablet 
     * @param newConfig
     * @see android.app.Activity#onConfigurationChanged(android.content.res.Configuration)
     **************************************************************************************/
    @Override
    public void onConfigurationChanged(Configuration newConfig) 
    {
      super.onConfigurationChanged(newConfig);
      setContentView(R.layout.main);
      showASBStatus();
    }
    
    
    /**************************************************************************************
     * createAndRunStatusThread
     * 
     * @param act main activity
     **************************************************************************************/
    public void createAndRunStatusThread(final Activity act)
    {
        m_bStatusThreadStop=false;
		m_statusThread = new Thread(new Runnable() 
	   	{
		   	public void run() 
		   	{
		   		while(m_bStatusThreadStop==false)
		   		{
		   			try
		   			{
		   				//anything touching the GUI has to run on the Ui thread
		   		    	act.runOnUiThread(new Runnable()
	   			    	{
	   			    		public void run()
	   			    		{
	   			    	    	//set indicator background color
	   			    	        TextView tvOpen =(TextView)findViewById(R.id.textViewOpen);
	   			    	        TextView tvClosed =(TextView)findViewById(R.id.textViewClosed);
	   			    	        if(m_Device.isDeviceOpen())
	   			    	        {
	   			    	        	tvOpen.setBackgroundColor(0xFF00E000);		//green
	   			    	        	tvClosed.setBackgroundColor(0xFF707070);	//gray
	   			    	        }
	   			    	        else
	   			    	        {
	   			    	        	tvOpen.setBackgroundColor(0xFF707070);		//gray
	   			    	        	tvClosed.setBackgroundColor(0xFFE00000);	//red
	   			    	        }
	   			    		}
	   			    	});

		   				Thread.sleep(200);
		   			}
		   			catch(InterruptedException e)
		   			{
		   				m_bStatusThreadStop = true;
		   				messageBox(act, "Exception in status thread: " + e.toString() + " - " + e.getMessage(), "createAndRunStatusThread Error");
		   			}
		   		}
		   	}
	   	});
		m_statusThread.start();
    }
    
    
    /**************************************************************************************
     * openButtonClicked
     * 
     * @param view
     **************************************************************************************/
    public void openButtonClicked(View view)
    {
    	ERROR_CODE err = ERROR_CODE.SUCCESS;
    	
    	EditText editTextIPAddress = (EditText)findViewById(R.id.editTextIPAddress);
    	Spinner spinnerPortType = (Spinner)findViewById(R.id.spinnerPortType);
    	String selectedItem = (String)spinnerPortType.getSelectedItem();
    	
    	if(selectedItem.equals("Ethernet"))
    	{
	    	//fill in some parameters
	    	m_DeviceParameters.PortType = PORT_TYPE.ETHERNET;
	    	m_DeviceParameters.IPAddress = editTextIPAddress.getText().toString();
   	    	m_DeviceParameters.PortNumber = 9100;
    	}
    	else if(selectedItem.equals("USB"))
    	{
	    	//fill in some parameters
	    	m_DeviceParameters.PortType = PORT_TYPE.USB;
	    	m_DeviceParameters.PortName = editTextIPAddress.getText().toString(); //if left empty, it takes first found compatible EPSON device
	    	m_DeviceParameters.ApplicationContext = this;
    	}
    	else
    	{
			err = ERROR_CODE.INVALID_PORT_TYPE;
    	}
    	
    	if(err==ERROR_CODE.SUCCESS)
    	{
	    	//set the parameters to the device 
	    	err = m_Device.setDeviceParameters(m_DeviceParameters);
	    	if(err!=ERROR_CODE.SUCCESS)
	    	{
				String errorString = EpsonCom.getErrorText(err);
	    		messageBox(this, errorString, "SampleCode: setDeviceParameters Error");
	    	}
	    	
	    	if(err==ERROR_CODE.SUCCESS)
	    	{
	    		//open the device
	    		err = m_Device.openDevice();
	        	if(err!=ERROR_CODE.SUCCESS)
	        	{
	    			String errorString = EpsonCom.getErrorText(err);
			   		Log.d("SampleCode", "Error from openDevice: " + errorString);
	        		messageBox(this, errorString, "SampleCode 0: openDevice Error");
	        	}
	    	}
	    	
	    	if(err==ERROR_CODE.SUCCESS)
	    	{
	    		//activate ASB sending
	    		err = m_Device.activateASB(true, true, true, true, true, true);
	        	if(err!=ERROR_CODE.SUCCESS)
	        	{
	    			String errorString = EpsonCom.getErrorText(err);
			   		Log.d("SampleCode", "Error from activateASB: " + errorString);
	        		messageBox(this, errorString, "SampleCode 1: openDevice Error");
	        	}
	    	}
    	}    	
    }

    
    /**************************************************************************************
     * closeButtonClicked
     * 
     * @param view
     **************************************************************************************/
    public void closeButtonClicked(View view)
    {
    	ERROR_CODE err = m_Device.closeDevice();
    	if(err!=ERROR_CODE.SUCCESS)
    	{
			String errorString = EpsonCom.getErrorText(err);
    		messageBox(this, errorString, "closeDevice Error");
    	}
    	
    	//reset the ASB indicators
        TextView textViewASBOnline =(TextView)findViewById(R.id.textViewASBOnline);
        TextView textViewASBCover =(TextView)findViewById(R.id.textViewASBCover);
        TextView textViewASBPaper =(TextView)findViewById(R.id.textViewASBPaper);
        TextView textViewASBSlip =(TextView)findViewById(R.id.textViewASBSlip);
        textViewASBOnline.setBackgroundColor(0xFF707070);	//gray
        textViewASBCover.setBackgroundColor(0xFF707070);	//gray
        textViewASBPaper.setBackgroundColor(0xFF707070);	//gray
		textViewASBSlip.setBackgroundColor(0xFF707070);		//gray
    	
    }
    
    /**************************************************************************************
     * printStringButtonClicked
     * 
     * @param view
     **************************************************************************************/
    public void printStringButtonClicked(View view)
    {
    	ERROR_CODE err = ERROR_CODE.SUCCESS;
    	
    	FONT font = FONT.FONT_A;
    	Boolean bold = false;
    	Boolean underlined = false;
    	Boolean doubleHeight = false;
    	Boolean doubleWidth = false;
    	
    	try
    	{
    		//get UI elements
    		EditText editTextPrintString = (EditText)findViewById(R.id.editTextPrintString);
            RadioGroup radioGroupFont = (RadioGroup)findViewById(R.id.radioGroupFont);
            RadioGroup radioGroupDoublePrint = (RadioGroup)findViewById(R.id.radioGroupDoublePrint);
            RadioGroup radioGroupAlignment = (RadioGroup)findViewById(R.id.radioGroupAlignment);
            CheckBox checkBoxBold  = (CheckBox)findViewById(R.id.checkBoxBold);
            CheckBox checkBoxUnderlined  = (CheckBox)findViewById(R.id.checkBoxUnderlined);
    		
			//set font
	    	switch(radioGroupFont.getCheckedRadioButtonId())
	    	{
	    		case R.id.radioFontA:
	    	    	font = FONT.FONT_A;
	    	    	break;
	    		case R.id.radioFontB:
	    	    	font = FONT.FONT_B;
	    	    	break;
	    	}
	    	
	    	//set boldness
	    	if(checkBoxBold.isChecked())
	    	{
	            bold = true;
	    	}
	    	else
	    	{
	            bold = false;
	    	}
	    		
	    	//set underlined
	    	if(checkBoxUnderlined.isChecked())
	    	{
	            underlined = true;
	    	}
	    	else
	    	{
	            underlined = false;
	    	}
	    	
			//set double print
	    	switch(radioGroupDoublePrint.getCheckedRadioButtonId())
	    	{
	    		case R.id.radioX1Y1:
	    	        doubleHeight = false;
	    	        doubleWidth = false;
	    	    	break;
	    		case R.id.radioX2Y1:
	    	        doubleHeight = false;
	    	        doubleWidth = true;
	    	    	break;
	    		case R.id.radioX1Y2:
	    	        doubleHeight = true;
	    	        doubleWidth = false;
	    	    	break;
	    		case R.id.radioX2Y2:
	    	        doubleHeight = true;
	    	        doubleWidth = true;
	    	    	break;
	    	}
	    	
	    	if(m_Device.isDeviceOpen()==true)
	    	{
	    		//set print alignment
	        	switch(radioGroupAlignment.getCheckedRadioButtonId())
	        	{
	        		case R.id.radioLeft:
	        			err = m_Device.selectAlignment(ALIGNMENT.LEFT);
	    	    		if(err!=ERROR_CODE.SUCCESS)
	    	    		{
	    	    			String errorString = EpsonCom.getErrorText(err);
	    	        		messageBox(this, errorString, "printString Error");
	    	    		}
	        			break;
	        		case R.id.radioCenter:
	        			err = m_Device.selectAlignment(ALIGNMENT.CENTER);
	    	    		if(err!=ERROR_CODE.SUCCESS)
	    	    		{
	    	    			String errorString = EpsonCom.getErrorText(err);
	    	        		messageBox(this, errorString, "printString Error");
	    	    		}
	        			break;
	        		case R.id.radioRight:
	        			err = m_Device.selectAlignment(ALIGNMENT.RIGHT);
	    	    		if(err!=ERROR_CODE.SUCCESS)
	    	    		{
	    	    			String errorString = EpsonCom.getErrorText(err);
	    	        		messageBox(this, errorString, "printString Error");
	    	    		}
	        			break;
	        	}	        	
	        	
	        	if(err==ERROR_CODE.SUCCESS)
	        	{
		        	//print string
		    		String sendString = editTextPrintString.getText().toString();
		    		err = m_Device.printString(sendString, font, bold, underlined, doubleHeight, doubleWidth);
		    		if(err!=EpsonCom.ERROR_CODE.SUCCESS)
		    		{
		    			String errorString = EpsonCom.getErrorText(err);
		        		messageBox(this, errorString, "printString Error");
		    		}
	        	}
	    	}
	    	else
	    	{
	    		messageBox(this, "Device is not open", "printString Error");
	    	}
	    }
	    catch(Exception e)
	    {
			messageBox(this, "Exception: " + e.toString() + " - " + e.getMessage(), "printString Error");
	    }
    	
    }    

    /**************************************************************************************
     * printStationRadioGroupClicked
     * 
     * @param selectedItem
     **************************************************************************************/
    public void printStationRadioGroupClicked(int selectedItem)
    {
    	ERROR_CODE err = ERROR_CODE.SUCCESS;
    	
        try
        {
	    	if(m_Device.isDeviceOpen()==true)
	    	{
				//set print station
		    	switch(selectedItem)
		    	{
		    		case R.id.radioReceipt:
		    			err = m_Device.selectReceiptPaper();
			    		if(err!=EpsonCom.ERROR_CODE.SUCCESS)
			    		{
			    			String errorString = EpsonCom.getErrorText(err);
			        		messageBox(this, errorString, "setPrintStation Error");
			    		}
		    			break;
		    		case R.id.radioSlipFront:
		    			err = m_Device.selectSlipPaper(PAPERSIDE.FRONT);
			    		if(err!=EpsonCom.ERROR_CODE.SUCCESS)
			    		{
			    			String errorString = EpsonCom.getErrorText(err);
			        		messageBox(this, errorString, "setPrintStation Error");
			    		}
		    			break;
		    		case R.id.radioSlipBack:
		    			err = m_Device.selectSlipPaper(PAPERSIDE.BACK);
			    		if(err!=EpsonCom.ERROR_CODE.SUCCESS)
			    		{
			    			String errorString = EpsonCom.getErrorText(err);
			        		messageBox(this, errorString, "setPrintStation Error");
			    		}
		    			break;
		    	}
	    	}
	    	else
	    	{
	    		messageBox(this, "Device is not open", "setPrintStation Error");
	    	}
        }
        catch(Exception e)
        {
    		messageBox(this, "Exception:" + e.getMessage(), "setPrintStation Error");
        }
    	
    	
    }
    
    /**************************************************************************************
     * cancelWaitForSlipButtonClicked
     * 
     * @param view
     **************************************************************************************/
    public void cancelWaitForSlipButtonClicked(View view)
    {
    	ERROR_CODE err = ERROR_CODE.SUCCESS;

    	try
        {
	    	if(m_Device.isDeviceOpen()==true)
	    	{
	    		err = m_Device.cancelWaitForSlip();
	    		if(err!=EpsonCom.ERROR_CODE.SUCCESS)
	    		{
	    			String errorString = EpsonCom.getErrorText(err);
	        		messageBox(this, errorString, "cancelWaitForSlip Error");
	    		}
	    		else
	    		{
	    			//reset the print station radio buttons
	                RadioGroup radioGroupPrintStation = (RadioGroup)findViewById(R.id.radioGroupPrintStation);
	                radioGroupPrintStation.check(R.id.radioReceipt);
	    		}
	    	}
	    	else
	    	{
	    		messageBox(this, "Device is not open", "cancelWaitForSlip Error");
	    	}
        }
        catch(Exception e)
        {
    		messageBox(this, "Exception:" + e.getMessage(), "cancelWaitForSlip Error");
        }
    }    

    /**************************************************************************************
     * ejectSlipButtonClicked
     * 
     * @param view
     **************************************************************************************/
    public void ejectSlipButtonClicked(View view)
    {
    	ERROR_CODE err = ERROR_CODE.SUCCESS;

    	try
        {
	    	if(m_Device.isDeviceOpen()==true)
	    	{
	    		err = m_Device.printPage(0);
	    		if(err!=EpsonCom.ERROR_CODE.SUCCESS)
	    		{
	    			String errorString = EpsonCom.getErrorText(err);
	        		messageBox(this, errorString, "ejectSlip Error");
	    		}	    		
	    		else
	    		{
	    			//reset the print station radio buttons
	                RadioGroup radioGroupPrintStation = (RadioGroup)findViewById(R.id.radioGroupPrintStation);
	                radioGroupPrintStation.check(R.id.radioReceipt);
	    		}
	    	}
	    	else
	    	{
	    		messageBox(this, "Device is not open", "ejectSlip Error");
	    	}
        }
        catch(Exception e)
        {
    		messageBox(this, "Exception:" + e.getMessage(), "ejectSlip Error");
        }
    }    
    
    /**************************************************************************************
     * curPaperButtonClicked
     * 
     * @param view
     **************************************************************************************/
    public void cutPaperButtonClicked(View view)
    {
    	ERROR_CODE err = ERROR_CODE.SUCCESS;

    	try
        {
	    	if(m_Device.isDeviceOpen()==true)
	    	{
	    		err = m_Device.cutPaper();
	    		if(err!=EpsonCom.ERROR_CODE.SUCCESS)
	    		{
	    			String errorString = EpsonCom.getErrorText(err);
	        		messageBox(this, errorString, "cutPaper Error");
	    		}	    		
	    	}
	    	else
	    	{
	    		messageBox(this, "Device is not open", "cutPaper Error");
	    	}
        }
        catch(Exception e)
        {
    		messageBox(this, "Exception:" + e.getMessage(), "cutPaper Error");
        }
    }    
    
    /**************************************************************************************
     * readMICRScanButtonClicked
     * 
     * @param view
     **************************************************************************************/
    public void readMICRScanButtonClicked(View view)
    {
    	//define standard parameters for scanning
    	ERROR_CODE err = ERROR_CODE.SUCCESS;
    	BITDEPTH bitDepth = BITDEPTH.GRAYSCALE;
    	IMAGEPROCESSING imgProcessing = IMAGEPROCESSING.NONE;
    	IMAGEFORMAT imgFormat = IMAGEFORMAT.JPEG_HIGH;
    	int threshold = 0;

    	try
        {
            //delete current MICR text
//	    	this.runOnUiThread(
//				new Runnable()
//				{
//					public void run()
//					{
						TextView textViewMICR = (TextView)findViewById(R.id.textViewMICRString);
						textViewMICR.setText("");
//					}
//				});

			//show progress dialog
			m_progressDialog = ProgressDialog.show(this, "Please Wait", "Scanning...");

    		if(m_Device.isDeviceOpen()==true)
	    	{
	    		//set image parameters
	        	err = m_Device.setImageParameters(bitDepth, imgProcessing, threshold);
	    		if(err!=EpsonCom.ERROR_CODE.SUCCESS)
	    		{
	    			String errorString = EpsonCom.getErrorText(err);
	        		messageBox(this, errorString, "readMICR+Scan Error");
	    		}

	    		//set image format
	        	if(err==EpsonCom.ERROR_CODE.SUCCESS)
	        	{
	        		err = m_Device.selectImageFormat(imgFormat);
		    		if(err!=EpsonCom.ERROR_CODE.SUCCESS)
		    		{
		    			String errorString = EpsonCom.getErrorText(err);
		        		messageBox(this, errorString, "readMICR+Scan Error");
		    		}
	        	}
	        	
	    		//do the reading and scanning
	        	if(err==EpsonCom.ERROR_CODE.SUCCESS)
	        	{
	        		err = m_Device.readMICRScanCheck(MICR_FONT.E13B, false, 0);
		    		if(err!=EpsonCom.ERROR_CODE.SUCCESS)
		    		{
		    			String errorString = EpsonCom.getErrorText(err);
		        		messageBox(this, errorString, "readMICR+Scan Error");
		    		}
	        	}
	    	}
	    	else
	    	{
	    		messageBox(this, "Device is not open", "readMICR+Scan Error");
	    	}
        }
        catch(Exception e)
        {
    		messageBox(this, "Exception:" + e.getMessage(), "readMICR+Scan Error");
        }
    }    

    
    
    
    /**************************************************************************************
	 * CallbackMethod
	 * This, method will be called by the EpsonCom library when new data arrives from the device
	 * @param cbInfo object of type CallbackInfo to convey information into the callback
	 * @return EpsonCom.ERROR_CODE
     **************************************************************************************/
	public ERROR_CODE CallbackMethod(EpsonComCallbackInfo cbInfo)
	{
    	EpsonCom.ERROR_CODE retval = EpsonCom.ERROR_CODE.SUCCESS;
		
    	try
    	{
	    	switch(cbInfo.ReceivedDataType)
	    	{
	    		case GENERAL:
	    			//do nothing, ignore any general incoming data 
	    			break;
	    		case ASB:	//new ASB data came in
	    			receiveAndShowASBData();
	    			break;
	    		case MICR:	//new MICR data came in
	    			receiveAndShowMICRData();
	    			break;
	    		case IMAGE:	//new image data came in
	    			receiveAndShowImageData();

	    			//kill progress dialog
	    	    	if(m_progressDialog!=null)
	    	    	{
	    	    		m_progressDialog.dismiss();
	    	    	}
	    	}
    	}
    	catch(Exception e)
    	{
    		messageBox(this, "callback method threw exception: " + e.toString() + " - " + e.getMessage(), "Callback Error");
    	}
    	
    	return retval;
	}
    
    
    /**************************************************************************************
     * receiveAndShowASBData
     * 
     **************************************************************************************/
    private void receiveAndShowASBData()
    {
		//retrieve current ASB data and show it
		m_ASBData = m_Device.getASB();
		showASBStatus();
    }

    
    /**************************************************************************************
     * showASBStatus
     * 
     **************************************************************************************/
    private void showASBStatus()
    {
		try
		{
	    	this.runOnUiThread(
				new Runnable()
				{
					public void run()
					{
						//get the status indicators from the GUI
		    	        TextView textViewASBOnline =(TextView)findViewById(R.id.textViewASBOnline);
		    	        TextView textViewASBCover =(TextView)findViewById(R.id.textViewASBCover);
		    	        TextView textViewASBPaper =(TextView)findViewById(R.id.textViewASBPaper);
		    	        TextView textViewASBSlip =(TextView)findViewById(R.id.textViewASBSlip);

						if(m_ASBData!=null)
						{
							//light up indicators
							if(m_ASBData.Online)
							{
			    	        	textViewASBOnline.setBackgroundColor(0xFF00E000);	//green
							}
							else
							{
			    	        	textViewASBOnline.setBackgroundColor(0xFFE00000);	//red
							}
							if(m_ASBData.CoverOpen)
							{
								textViewASBCover.setBackgroundColor(0xFFE00000);	//red
							}
							else
							{
			    	        	textViewASBCover.setBackgroundColor(0xFF00E000);	//green
							}
							if(m_ASBData.PaperOut==true)
							{
								textViewASBPaper.setBackgroundColor(0xFFE00000);	//red
							}
							else
							{
								if(m_ASBData.PaperNearEnd==true)
								{
									textViewASBPaper.setBackgroundColor(0xFFE0E000);	//yellow
								}
								else
								{
									textViewASBPaper.setBackgroundColor(0xFF00E000);	//green
								}
							}
							if(m_ASBData.SlipSelectedAsActiveSheet)
							{
								textViewASBSlip.setBackgroundColor(0xFF00E000);	//green
							}
							else
							{
								textViewASBSlip.setBackgroundColor(0xFF707070);	//gray
							}
						}
					} //public void run()
				}); //this.runOnUiThread(
    	}
		catch(Exception e)
		{
    		messageBox(this, "receiveAndShowASBData threw exception: " + e.toString() + " - " + e.getMessage(), "ReceiveAndShowASBData Error");
		}
    }
    
    
    /**************************************************************************************
     * receiveAndShowMICRData
     * 
     **************************************************************************************/
    private void receiveAndShowMICRData()
    {
		try
		{
	    	this.runOnUiThread(
				new Runnable()
				{
					public void run()
					{
						//get the text box
						TextView textViewMICR = (TextView)findViewById(R.id.textViewMICRString);

						//retrieve MICR string
						String MICRString = m_Device.getMICRString();
						if(MICRString!=null)
						{
							MICRString = MICRString.replace(" ", ""); //to prevent line break in text view
							textViewMICR.setText(MICRString);
						}
					}
				});
		}
		catch(Exception e)
		{
    		messageBox(this, "receiveAndShowMICRData threw exception: " + e.toString() + " - " + e.getMessage(), "receiveAndShowMICRData Error");
		}
    }
    
    /**************************************************************************************
     * receiveAndShowImageData
     * 
     **************************************************************************************/
    private void receiveAndShowImageData()
    {
		byte[] receivedBytes = null;
		
		try
		{
			//get image data
			receivedBytes = m_Device.getImageDataBytes();
			
			if(receivedBytes!=null && receivedBytes.length>0)
			{
				//convert the image data into a bitmap
				InputStream in = new ByteArrayInputStream(receivedBytes);
				BufferedInputStream buf = new BufferedInputStream(in);
				Bitmap bm = BitmapFactory.decodeStream(buf);
				
				//rotate bitmap by 90 degrees and show it 
				if(bm!=null)
				{
					Matrix mt = new Matrix(); 
					mt.postRotate(90);
					Bitmap bm2 = Bitmap.createBitmap(bm, 0, 0, bm.getWidth(), bm.getHeight(), mt, true); 
					showBitmap(bm2);
				}
				else
				{
					messageBox(this, "Error: bitmap bm is null after decoding from stream","message");
				}
			}
			else
			{
	    		messageBox(this, "receivedBytes is null or empty", "receiveAndShowImageData Error");
			}
		}
		catch(Exception e)
		{
    		messageBox(this, "receiveAndShowImageData threw exception: " + e.toString() + " - " + e.getMessage(), "receiveAndShowImageData Error");
		}
    }

    
    /**************************************************************************************
     * show bitmap
     * 
     * @param text
     **************************************************************************************/
    private void showBitmap(final Bitmap bm)
    {
    	try
    	{
	    	this.runOnUiThread(
    			new Runnable()
    			{
    				public void run()
    				{
						//get the image view from the GUI
		    	        ImageView imageViewCheck =(ImageView)findViewById(R.id.imageViewCheck);
    					imageViewCheck.setImageBitmap(bm);
    				}
    			});
    	}
		catch(Exception e)
		{
    		messageBox(this, "showBitmap threw exception: " + e.toString() + " - " + e.getMessage(), "showBitmap Error");
		}
    }
    
    
    
    /**************************************************************************************
     * messageBox
     * Shows a standard message box with OK button. 
     * Note: the program execution does not stop!
     * @param context
     * @param message
     * @param title
     **************************************************************************************/
    public void messageBox(final Context context, final String message, final String title)
    {
    	this.runOnUiThread(
			new Runnable()
			{
				public void run()
				{
					final AlertDialog alertDialog = new AlertDialog.Builder(context).create();
					alertDialog.setTitle(title);
					alertDialog.setMessage(message);
					alertDialog.setButton("OK", new OnClickListener()
					{
						@Override
						public void onClick(DialogInterface dialog, int which) 
						{               
							alertDialog.cancel();    	
						}
					});    	
					alertDialog.show();    	
				}
			}
    	);
    }
    
}